;****************************************************************************
;
;                         PACKET TRANSMITING VERSION 1.0
;
; BY REMI BILODEAU NOVEMBER 11, 1994
;
; 80C51 VERSION
;
;****************************************************************************

 
;****************************************************************************
;
;                  TRANSMIT PACKET, GET IT FROM BYTE STREAM
;
; CPU cycle useable for different baud rate:
; 300  baud = 3072 cycles
; 600  baud = 1536 cycles
; 1200 baud = 768  cycles
;
; Function description:
; PacketTXINIT   - Initialize packet transmitter
; PacketTRANSMIT - Receive packet, put it in RX queue data block.
; MakeTRANSIT    - Make transition for a certain bit length
;
;****************************************************************************



 
;****************************************************************************
;
;                            EQUIVALENCE FOR PTX
;
;****************************************************************************
TXQhandle       EQU     12h     ; Handle of TX queue data block 
TXtimerCnt      EQU     13h     ; Timer value on last transition
TXDelay         EQU     14h     ; TXDELAY: number of flag to send when transmit
SlotCount       EQU     15h     ; Slottime decounter
SlotTime        EQU     16h     ; SLOTTIME: Interval of time to wait for TX
Persist         EQU     17h     ; PERSIST: Chance to transmit at interval

TXQempty        EQU     4       ; FLAG: if set, TX queue have at least one blk.

TXQsize         EQU     1024    ; Maximum size of TX queue data block

Abit0           EQU     0E0h    ; Bit 0 of accumulator
Abit1           EQU     0E1h    ; Bit 1 of accumulator
Abit2           EQU     0E2h    ; Bit 2 of accumulator
Abit3           EQU     0E3h    ; Bit 3 of accumulator
Abit4           EQU     0E4h    ; Bit 4 of accumulator
Abit5           EQU     0E5h    ; Bit 5 of accumulator
Abit6           EQU     0E6h    ; Bit 6 of accumulator
Abit7           EQU     0E7h    ; Bit 7 of accumulator


 
;****************************************************************************
;
;            INITIALIZE MEMORY BUFFER FOR TX QUEUE AND OTHER STUFF
;
; Entry:  None
; Return: None
; 
;****************************************************************************
PacketTXINIT:
        MOV     dptr,#TXQsize           ; Reserve memory for TX queue block
        CALL    MemALLOC
        MOV     TXQhandle,a
        CLR     a                       ; Clear buffer (size = 0)
        MOVX    @dptr,a
        INC     dptr
        MOVX    @dptr,a
        RET


 
;****************************************************************************
;
;                      SEND PACKET SYNCRONIZATION FLAGS
;
; Entry:  A is Number of flag to send 1-255
; Return: None
;
; R0 destroyed
;
;****************************************************************************
SendFlag:
        MOV     R0,a            ; Set number of flag to send in R0
SF1:    MOV     a,#1            ; Step one bit
        CALL    MakeTRANSIT     ; Transit for bit 0 (flag is 01111110 NRZI)
        MOV     a,#7            ; Step seven bit
        CALL    MakeTRANSIT     ; Transit for bit 7
        DJNZ    R0,SF1
        RET

 
;****************************************************************************
;
;                          SEND PACKET TO PACKET PORT
;
; Entry:  DPTR to packet byte stream
; Return: DPTR to next packet entry
;
; Register usage:
; R1:R0  =  Byte remaining to send
; R3     =  Byte currently transmitted
; R5     =  Bit left in current transmitted byte
;
;****************************************************************************
SendPACKETDATA:

        MOV     R5,#0           ; R5 = bit left in current byte
        MOVX    a,@dptr         ; R1:R0 = Number of byte to send
        MOV     R0,a
        INC     dptr
        MOVX    a,@dptr
        MOV     R1,a
        INC     dptr

SendP0: MOV     R4,#0           ; R4 = sequencial bit counter
SendP1: CJNE    R5,#0,SendP4    ; any bit left in current byte?
        CJNE    R0,#0,SendP3    ; no, if data are all tranfered?
        CJNE    R1,#0,SendP3
        CJNE    R4,#5,SendP2    ; Sequencial bit counter reach 5?
        MOV     a,#6
        CALL    MakeTRANSIT
SendP2: RET

SendP3: DEC     R0              ; Decrementing remaining byte counter
        CJNE    R0,#255,SendPa  
        DEC     R1
SendPa: MOVX    a,@dptr         ; R3 = current byte
        MOV     R3,a
        INC     dptr
        MOV     R5,#7           ; 7 bit left in new byte
        SJMP    SendP5

SendP4: MOV     a,R3            ; shift for next bit check
        RR      a
        MOV     R3,a
        DEC     R5              ; least one bit left

SendP5: INC     R4              ; increment sequencial bit counter

        MOV     a,R3            ; loop if counter<6 and bit is 1
        JNB     Abit0,SendP7
        CJNE    R4,#6,SendP1

SendP7: CJNE    R4,#6,SendP8    ; if count==6?
        INC     R5              ; yes, bit left++
        MOV     a,R3            ; invert rotation of bit
        RL      a
        MOV     R3,a

SendP8: MOV     a,R4            ; make transition of 'bit counter' lenght
        CALL    MakeTRANSIT
        JMP     SendP0
       

 
;****************************************************************************
;
;                      EMPTY ALL PACKET TRANSMISSION QUEUE
;
; Entry:  None
; Return: None
;
;****************************************************************************
PacketEMPTYQUEUE:
        CLR     TXQempty                ; TX queue are empty (after this func)
        MOV     a,SlotTime              ; Reset slotcount variable
        MOV     SlotCount,a

        MOV     a,TXQhandle             ; Get pointer to TX queue
        CALL    MemGETPOINTER           
        PUSH    dpl
        PUSH    dph
        MOVX    a,@dptr                 ; Check if any packet to send
        JNZ     TXQe0
        INC     dptr
        MOVX    a,@dptr
        DEC_DPTR
        JNZ     TXQe0
        JMP     TXQe2
        
TXQe0:  SETB    PTT                     ; key-up the PTT
        MOV     a,TH0
        MOV     TXtimerCnt,a            ; Reset timer value variable
        MOV     a,TXDELAY               ; Send starting flags
        CALL    SendFlag

TXQe1:  MOVX    a,@dptr                 ; Check if any other packet to send
        JNZ     TXQe1a
        INC     dptr
        MOVX    a,@dptr
        DEC_DPTR
        JNZ     TXQe1a
        JMP     TXQe2

TXQe1a: CALL    SendPACKETDATA          ; Send one data entry
        MOV     a,#2                    ; Send 2 flag for separator
        CALL    SendFlag
        JMP     TXQe1
        
TXQe2:  CLR     PTT                     ; release PTT
        POP     dph
        POP     dpl
        CLR     a                       ; Clear all TX queue
        MOVX    @dptr,a
        INC     dptr
        MOVX    @dptr,a
        RET

 
;****************************************************************************
;
;                       MAKE TRANSITION WITH BIT LENGTH
;
; Entry:  Bit length to wait for transition in A, from 1 to 7
; Return: None
; 
; Variable used:
; HBaud  -  Packet baud rate (baud rate are locked now to 1200)
;
; Information:
; baud rate accepted is 300,600 and 1200 bauds (1200 bauds is locked now)
;                                               
; A,B destroyed
; 
;****************************************************************************
MakeTRANSIT:
        MOV     b,#3            ; Calculate timer value to detect (1200 bauds)
        MUL     ab
        ADD     a,TXtimerCnt
        MOV     b,a
        
MT0:    XRL     a,TH0           ; Try to detect value (death time for TNC)
        JZ      MT1
        MOV     a,b
        JMP     MT0

MT1:    CPL     PktOUT          ; Invert output
        MOV     TXtimerCnt,b
        RET
