;****************************************************************************
;
;                          TIMER MANAGER VERSION 1.0
;
; BY REMI BILODEAU NOVEMBER 9, 1994
;
; 8051 VERSION
;
;****************************************************************************
 
;****************************************************************************
;
;                      SERIAL PORT MANAGER DESCRIPTION
;
; Memory requirement:
; None
;
; Function description:
; TimerINIT  -  Initialize timer for 80C51/80C52 CPU, 300Hz interrupt
; TimerINT   -  Interrupt driver for real time clock.
;
;****************************************************************************



 
;****************************************************************************
;
;                            EQUIVALENCE FOR SPM
;
;****************************************************************************
Tick10ms        EQU             18h             ; 10ms tick counter
Tick100ms       EQU             1Dh             ; 100ms tick counter
Tick1s          EQU             19h             ; Real time second counter
Tick1min        EQU             1Ah             ; Real time minute counter
Tick1hour       EQU             1Bh             ; Real time hour counter
Tick300Hz       EQU             1Ch             ; 1/3 tick counter
T1count         EQU             3Ch             ; T1 counter value (1 sec)
T2count         EQU             3Dh             ; T2 counter value (100ms)
T3count         EQU             3Eh             ; T3 counter value (1 min)


CPUTYPE         EQU             3               ; CPU type, 80C51/80C52
T1run           EQU             5               ; T1 AX25 timer run bit
T2run           EQU             6               ; T2 AX25 timer run bit
T3run           EQU             7               ; T3 AX25 timer run bit



 
;****************************************************************************
;
;                 INITIALIZE TIMER 2/IT0 FOR 16-BIT MODE, 300 HZ
;
; Entry:  None
; Return: None
;
;****************************************************************************
TimerINIT:
         
        MOV     Tick300Hz,#0
        MOV     Tick10ms,#0
        MOV     Tick1s,#0
        MOV     Tick1min,#0
        MOV     Tick1hour,#0

         
        MOV     R0,#255
        MOV     a,#8Bh
        MOV     @R0,a
        MOV     a,@R0
        XRL     a,#8Bh
        JZ      Is8052

         
        SETB    IT0                     ; Edge triggered interrupt
        SETB    EX0                     ; Enable external interrupt 0
        CLR     CPUTYPE                 ; CPU is 80C51
        RET

         
Is8052: MOV     T2CON,#00h             ; 16-bit auto-reload
        MOV     RCAP2L,#00h            ; 3.33ms interval (1/300 sec.)
        MOV     TL2,#00h
        MOV     RCAP2H,#0F4h
        MOV     TH2,#0F4h
        SETB    TR2                    ; timer run
        SETB    ET2                    ; enable interrupt
        SETB    CPUTYPE                ; CPU is 80C52
        RET


 
;****************************************************************************
;
;                      TIMER 0 INTERRUPT DRIVER (1/100 sec)
;
; -Reload timer
; -Increment real time clock
;
; Entry:  None
; Return: None
;
;****************************************************************************
TimerINT:
        PUSH    PSW
        PUSH    ACC

        CLR     TF2                      ; Clear Timer overflow (for 80C52)        

        INC     Tick300Hz                ; Increment 1/3 tick counter
        MOV     a,Tick300Hz
        XRL     a,#3
        JNZ     Tint0

        MOV     Tick300Hz,#0
        INC     Tick10ms                ; Increment 10ms tick counter
         
        JNB     TXQEmpty,Tint1          ; Decrement Slotcount if need
        MOV     a,SlotCount
        JZ      Tint1
        DEC     SlotCount     

Tint1:  MOV     a,Tick10ms
        XRL     a,#10
        JNZ     Tint0
        
        MOV     Tick10ms,#0
        INC     Tick100ms               ; Increment 100ms tick counter
         
        JNB     T2run,Tint2         
        MOV     a,T2count
        JZ      Tint2
        DEC     T2count     


Tint2:  MOV     a,Tick100ms
        XRL     a,#10
        JNZ     Tint0
        
        MOV     Tick100ms,#0
        INC     Tick1s                  ; Increment 1s tick counter
         
        JNB     T1run,Tint3
        MOV     a,T1count
        JZ      Tint3
        DEC     T1count     

Tint3:  MOV     a,Tick1s
        XRL     a,#60
        JNZ     Tint0
        
        MOV     Tick1s,#0
        INC     Tick1min                ; Increment 1 minute tick counter
         
        JNB     T3run,Tint4
        MOV     a,T3count
        JZ      Tint4
        DEC     T3count     

Tint4:  MOV     a,Tick1min
        XRL     a,#60
        JNZ     Tint0
        
        MOV     Tick1min,#0
        INC     Tick1hour               ; Increment 1 hour tick counter
        MOV     a,Tick1hour
        XRL     a,#24
        JNZ     Tint0
        MOV     Tick1hour,#0
                
Tint0:  POP     ACC
        POP     PSW
        RETI


