;****************************************************************************
;
;                  MICROREM TNC SOFTWARE DRIVER VERSION 1.0
;
; BY REMI BILODEAU  OCTOBER 13, 1994
;
; 8051' CPU Version
;
;****************************************************************************
         
;****************************************************************************
;
;                  DOCUMENTATION OF TNC SOFTWARE VERSION 1.0
;
;
;  I/O PORT #1 PINOUT:
;---------------------------------
;  P1.0  =  CONNECT led
;  P1.1  =  STATUS led
;  P1.2  =  Packet FSK in
;  P1.3  =  Packet FSK out
;  P1.4  =  DCD input
;  P1.5  =  PTT output
;  P1.6  =  CTS output (TNC ready)
;  P1.7  =  MAIL led
;
;****************************************************************************

         ;****************************************************************************
;
;                                INTERNAL RAM MAP
;
;  00-07 Register bank #0 - Common register 
;  08-0F Register bank #1 - Use for fast pushs all common register
;  10-17 Register bank #2
;  10h   HBaud          - Packet baud rate (doc in PACKETRX.ASM) - PACKETRX.ASM
;  11h   RXQHandle      - Handle of RX queue data block          - PACKETRX.ASM
;  12h   TXQHandle      - Handle of TX queue data block          - PACKETTX.ASM
;  13h   TXTimerCnt     - Timer value on last transition         - PACKETTX.ASM
;  14h   TXDELAY        - Number of HDLC flag to send at begin   - PACKETTX.ASM
;  15h   SlotCount      - Slottime counter                       - PACKETTX.ASM
;  16h   SlotTime       - Slottime interval, in 10ms tick        - PACKETTX.ASM
;  17h   Persist        - Persistance parameter, 0-255           - PACKETTX.ASM
;
;  18-1F register Bank #3
;  18h   Tick10ms       - 10ms tick counter (0 to 10)            - TIMER.ASM 
;  19h   Tick1s         - 1 second tick counter (1 to 59)        - TIMER.ASM
;  1Ah   Tick1min       - 1 minute tick counter (1 to 59)        - TIMER.ASM
;  1Bh   Tick1hour      - 1 hour tick counter (1 to 23)          - TIMER.ASM
;  1Ch   Tick300Hz      - Tick counter to make interrupt 300Hz   - TIMER.ASM
;  1Dh   Tick100ms      - 100ms tick counter (0 to 10)           - TIMER.ASM
;  1Eh   KissTXptr      - Pointer to TX buffer for KISS module   - KISS.ASM
;  1Fh   KissTXptr
;
;  20-2F Bit addressable segment
;  00h   TXemptyReady   - Set when TX are empty and ready        - SERIAL.ASM
;  01h   CRLFflag       - If set CR+LF are send else CR only     - SERIAL.ASM
;  02h   SerialECHO     - Received char are echoed to terminal   - SERIAL.ASM
;  03h   CPUTYPE        - Type of CPU, set=80C52 clear=80C51     - TIMER.ASM
;  04h   TXQEmpty       - Clear if TX queue empty else set       - PACKETTX.ASM
;  05h   T1run          - If set, T1 timer count-down (1sec)     - TIMER.ASM
;  06h   T2run          - If set, T2 timer count-down (100ms)    - TIMER.ASM
;  07h   T3run          - If set, T3 timer count-down (1min)     - TIMER.ASM  
;  ...
;  2Dh   KissStatus     - General status of KISS mode            - KISS.ASM
;  2Eh   KissTXsize     - Size of KISS TX buffer                 - KISS.ASM
;  2Fh   KissTXsize
;  7F free
;
;  30-7F Scratch pad area
;  30h   MemEntry       - 16-bit pointer to entry block table    - MEMORY.ASM
;  32h   SerRXptr       - Pointer to block of serial RX buffer   - SERIAL.ASM
;  34h   SerRXreadCnt   - Read pointer on serial RX buffer       - SERIAL.ASM 
;  35h   SerRXwriteCnt  - Write pointer on serial RX buffer      - SERIAL.ASM
;  36h   SerTXptr       - Pointer to block of serial TX buffer   - SERIAL.ASM
;  38h   SerTXreadCnt   - Read pointer on serial TX buffer       - SERIAL.ASM 
;  39h   SerTXwriteCnt  - Write pointer on serial TX buffer      - SERIAL.ASM
;  3Ah   SerTVALUE      - Serial timer value saving register     - SERIAL.ASM
;  3Bh   RandomPtr      - Random pointer on table                - CRC.ASM
;  3Ch   T1count        - AX25 T1 counter value in 1 sec tick    - TIMER.ASM
;  3Dh   T2count        - AX25 T2 counter value in 100ms tick    - TIMER.ASM
;  3Eh   T3count        - AX25 T3 counter value in 1 min tick    - TIMER.ASM
;  ...                                                         
;  60h-7Fh Stack use
;
;****************************************************************************

         ;****************************************************************************
;
;                     GENERAL EQUIVALENCE OF TNC SOFTWARE
;
;****************************************************************************

STACK           EQU             60h     ; Stack/SP pointer at 60h
CPUSTART        EQU             8000h   ; Program ROM start point

 
CONled          EQU     90h            ; P1.0 CONNECT led
STAled          EQU     91h            ; P1.1 STATUS led
PKTIn           EQU     92h            ; P1.2 Packet transmission input
PKTOut          EQU     93h            ; P1.3 Packet transmission output
DCD             EQU     94h            ; P1.4 Packet data carrier detect 
PTT             EQU     95h            ; P1.5 Packet push to talk switch
CTS             EQU     96h            ; P1.6 Clear to send serial control line
MAILled         EQU     97h            ; P1.7 MAIL led




        $MOD52
        org     CPUSTART+0000h

         
;****************************************************************************
;
;                           INTERRUPT VECTOR TABLE
;
;****************************************************************************

        LJMP    start                   ; Go to program start

        org     CPUSTART+0003h          ; INT0 vector, real time clock (80C51)
        AJMP    TimerINT                ; MUST be at 300 Hz for good timing
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP

        org     CPUSTART+000Bh          ; Timer #0 not used, packet timing
        RETI
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP

        org     CPUSTART+0013h          ; INT1 vector, wake-up from IDLE mode
        RETI
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP

        org     CPUSTART+001Bh          ; timer #1, serial baud rate, not used
        RETI
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP

        org     CPUSTART+0023h          ; Serial port interrupt
        LJMP    SerialINT    
        NOP
        NOP
        NOP
        NOP
        NOP

        org     CPUSTART+002Bh          ; TIMER #2, real time clock (80C52)
        AJMP    TimerINT



         
;****************************************************************************
;
;            TNC BEGIN HERE, INITIALIZE ALL SOFTWARE AND HARDWARE
;
;****************************************************************************

Start:  MOV     SP,#STACK               ; Setup stack position
        MOV     P1,#14h                 ; Setup I/O port (input pin set to '1')
        SETB    EA                      ; Enable interrupt
        SETB    EX1                     ; Enable IDLE state external wake-up

        CALL    TimerINIT               ; Init timer 2/IT0 for 300Hz interrupt
        CALL    MemINIT                 ; Initialize dynamic memory manager
        CALL    SerialINIT              ; Init serial port and his buffer
        CALL    PacketRXINIT            ; Init packet receiver
        CALL    PacketTXINIT            ; Init packet transmitter

         
;****************************************************************************
;
;                           DEFAULT PARAMETER SETTING
; 
        
        SETB    CRLFflag        ; CRLF are send when CR are send trough serial
        SETB    SerialECHO      ; Received char are echoed to terminal 
                                ; CR+LF conversion are maked here.
        MOV     HBaud,#2        ; default is 1200 bauds
        MOV     TXDELAY,#70     ; Send 22 flag before transmit packet
        MOV     SlotTime,#10    ; 100ms slottime
        MOV     SlotCount,#10   ; Slotcount are setup'ed
        MOV     Persist,#63     ; persistance is 25%
        CLR     TXQempty        ; TX queue are empty
        CLR     T0              ; Enable status leds
        CLR     T1run           ; Clear AX25 timers run bit
        CLR     T2run
        CLR     T3run


allo:   CALL    KISSmain                ; Enter KISS mode        
        MOV     dptr,#string
        CALL    SerialSTRING
        JMP     Allo


string: db '*** Out of kiss mode ***',0


        $INCLUDE(TIMER.ASM)
        $INCLUDE(MEMORY.ASM)
        $INCLUDE(SERIAL.ASM)
        $INCLUDE(PACKETRX.ASM)
        $INCLUDE(PACKETTX.ASM)
        $INCLUDE(CRC.ASM)
        $INCLUDE(KISS.ASM)

END

